package com.nisum.employeeservice.controller;

import com.nisum.employeeservice.model.Employee;
import com.nisum.employeeservice.service.EmployeeService;
import com.nisum.employeeservice.exception.ResourceNotFoundException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

@CrossOrigin(origins = "*")
@RestController
@RequestMapping("/api/v1")
public class EmployeeController {

    @Autowired
    EmployeeService employeeService;

    //read all
    @GetMapping("/employees")
    public List<Employee> getAllEmployees(){
        return employeeService.findAllEmployees();
    }

    //read one
    @GetMapping("/employees/{id}")
    public ResponseEntity<Employee> getEmployeeById(@PathVariable("id") Long employeeId) throws ResourceNotFoundException {
        Employee employee = employeeService.getEmployeeById(employeeId)
                .orElseThrow(()-> new ResourceNotFoundException("Employee by Id not found." + employeeId));
        return ResponseEntity.ok().body(employee);
    }

    //create
    @PostMapping("/employees")
    public Employee createEmployee(@RequestBody Employee employee){
        return employeeService.addEmployee(employee);
    }

    //update
    @PutMapping("/employees/{id}")
    public ResponseEntity<Employee> updateEmployee(@PathVariable("id") Long employeeId, @RequestBody @Valid Employee newEmployee) throws ResourceNotFoundException{
        Employee oldEmployee = employeeService.getEmployeeById(employeeId)
                .orElseThrow(()-> new ResourceNotFoundException("Could not find employee with that Id." + employeeId));
        employeeService.updateEmployee(oldEmployee, newEmployee);
        return ResponseEntity.ok().body(newEmployee);
    }

    //destroy
    @DeleteMapping("/employees/{id}")
    public ResponseEntity<Employee> deleteEmployee(@PathVariable("id") Long employeeId) throws ResourceNotFoundException{
        Employee employee = employeeService.getEmployeeById(employeeId)
                .orElseThrow(()-> new ResourceNotFoundException("No employee by that name." + employeeId));
        employeeService.deleteEmployee(employee);
        return ResponseEntity.ok().body(employee);
    }
}
